@@ -1,6 +1,6 @@
 /* mpf_pow_ui -- Compute b^e.
 
-Copyright 1998, 1999, 2001 Free Software Foundation, Inc.
+Copyright 1998, 1999, 2001, 2012 Free Software Foundation, Inc.
 
 This file is part of the GNU MP Library.
 
@@ -24,18 +24,18 @@ void
 mpf_pow_ui (mpf_ptr r, mpf_srcptr b, unsigned long int e)
 {
   mpf_t b2;
-  unsigned long int e2;
 
   mpf_init2 (b2, mpf_get_prec (r));
   mpf_set (b2, b);
-  mpf_set_ui (r, 1);
 
   if ((e & 1) != 0)
-    mpf_set (r, b2);
-  for (e2 = e >> 1; e2 != 0; e2 >>= 1)
+    mpf_set (r, b);
+  else
+    mpf_set_ui (r, 1);
+  while (e >>= 1)
     {
       mpf_mul (b2, b2, b2);
-      if ((e2 & 1) != 0)
+      if ((e & 1) != 0)
 	mpf_mul (r, r, b2);
     }
 
