@@ -23,35 +23,39 @@ along with the GNU MP Library.  If not, see http://www.gnu.org/licenses/.  */
 
 
 #define MOD_1_NORM_THRESHOLD                 0  /* always */
-#define MOD_1_UNNORM_THRESHOLD               4
-#define MOD_1N_TO_MOD_1_1_THRESHOLD         14
-#define MOD_1U_TO_MOD_1_1_THRESHOLD          6
-#define MOD_1_1_TO_MOD_1_2_THRESHOLD         0
-#define MOD_1_2_TO_MOD_1_4_THRESHOLD        20
-#define PREINV_MOD_1_TO_MOD_1_THRESHOLD     26
+#define MOD_1_UNNORM_THRESHOLD               3
+#define MOD_1N_TO_MOD_1_1_THRESHOLD          7
+#define MOD_1U_TO_MOD_1_1_THRESHOLD          3
+#define MOD_1_1_TO_MOD_1_2_THRESHOLD        24
+#define MOD_1_2_TO_MOD_1_4_THRESHOLD         0  /* never mpn_mod_1s_2p */
+#define PREINV_MOD_1_TO_MOD_1_THRESHOLD     10
 #define USE_PREINV_DIVREM_1                  1  /* native */
+#define DIV_QR_2_PI2_THRESHOLD           MP_SIZE_T_MAX  /* never */
 #define DIVEXACT_1_THRESHOLD                 0  /* always (native) */
-#define BMOD_1_TO_MOD_1_THRESHOLD           28
+#define BMOD_1_TO_MOD_1_THRESHOLD           24
 
 #define MUL_TOOM22_THRESHOLD                28
 #define MUL_TOOM33_THRESHOLD                85
-#define MUL_TOOM44_THRESHOLD               148
-#define MUL_TOOM6H_THRESHOLD               204
+#define MUL_TOOM44_THRESHOLD               142
+#define MUL_TOOM6H_THRESHOLD               258
 #define MUL_TOOM8H_THRESHOLD               309
 
 #define MUL_TOOM32_TO_TOOM43_THRESHOLD      85
 #define MUL_TOOM32_TO_TOOM53_THRESHOLD      99
-#define MUL_TOOM42_TO_TOOM53_THRESHOLD      93
-#define MUL_TOOM42_TO_TOOM63_THRESHOLD     101
+#define MUL_TOOM42_TO_TOOM53_THRESHOLD      97
+#define MUL_TOOM42_TO_TOOM63_THRESHOLD     102
+#define MUL_TOOM43_TO_TOOM54_THRESHOLD     144
 
 #define SQR_BASECASE_THRESHOLD               0  /* always (native) */
 #define SQR_TOOM2_THRESHOLD                 50
-#define SQR_TOOM3_THRESHOLD                 87
-#define SQR_TOOM4_THRESHOLD                208
-#define SQR_TOOM6_THRESHOLD                306
+#define SQR_TOOM3_THRESHOLD                 83
+#define SQR_TOOM4_THRESHOLD                216
+#define SQR_TOOM6_THRESHOLD                318
 #define SQR_TOOM8_THRESHOLD                430
 
-#define MULMOD_BNM1_THRESHOLD               18
+#define MULMID_TOOM42_THRESHOLD             56
+
+#define MULMOD_BNM1_THRESHOLD               17
 #define SQRMOD_BNM1_THRESHOLD               19
 
 #define MUL_FFT_MODF_THRESHOLD             888  /* k = 6 */
@@ -99,9 +103,9 @@ along with the GNU MP Library.  If not, see http://www.gnu.org/licenses/.  */
     {   1151,11}, {   2303,12}, {   1215,11}, {   2431,13}, \
     {   8192,14}, {  16384,15}, {  32768,16} }
 #define MUL_FFT_TABLE3_SIZE 167
-#define MUL_FFT_THRESHOLD                 7808
+#define MUL_FFT_THRESHOLD                 7552
 
-#define SQR_FFT_MODF_THRESHOLD             786  /* k = 6 */
+#define SQR_FFT_MODF_THRESHOLD             666  /* k = 6 */
 #define SQR_FFT_TABLE3                                      \
   { {    786, 6}, {     25, 7}, {     13, 6}, {     27, 7}, \
     {     15, 6}, {     31, 7}, {     17, 6}, {     35, 7}, \
@@ -149,37 +153,44 @@ along with the GNU MP Library.  If not, see http://www.gnu.org/licenses/.  */
     {   1215,11}, {   2431,13}, {   8192,14}, {  16384,15}, \
     {  32768,16} }
 #define SQR_FFT_TABLE3_SIZE 177
-#define SQR_FFT_THRESHOLD                 7552
+#define SQR_FFT_THRESHOLD                 7040
 
-#define MULLO_BASECASE_THRESHOLD            10
-#define MULLO_DC_THRESHOLD                  50
+#define MULLO_BASECASE_THRESHOLD            11
+#define MULLO_DC_THRESHOLD                  35
 #define MULLO_MUL_N_THRESHOLD            13463
 
-#define DC_DIV_QR_THRESHOLD                 60
-#define DC_DIVAPPR_Q_THRESHOLD             333
-#define DC_BDIV_QR_THRESHOLD                82
-#define DC_BDIV_Q_THRESHOLD                268
+#define DC_DIV_QR_THRESHOLD                 41
+#define DC_DIVAPPR_Q_THRESHOLD             214
+#define DC_BDIV_QR_THRESHOLD                41
+#define DC_BDIV_Q_THRESHOLD                148
 
-#define INV_MULMOD_BNM1_THRESHOLD           62
-#define INV_NEWTON_THRESHOLD               284
-#define INV_APPR_THRESHOLD                 290
+#define INV_MULMOD_BNM1_THRESHOLD           77
+#define INV_NEWTON_THRESHOLD               204
+#define INV_APPR_THRESHOLD                 204
 
-#define BINV_NEWTON_THRESHOLD              264
-#define REDC_1_TO_REDC_N_THRESHOLD          86
+#define BINV_NEWTON_THRESHOLD              230
+#define REDC_1_TO_REDC_N_THRESHOLD          59
 
-#define MU_DIV_QR_THRESHOLD               1858
-#define MU_DIVAPPR_Q_THRESHOLD            1718
-#define MUPI_DIV_QR_THRESHOLD              114
-#define MU_BDIV_QR_THRESHOLD              1387
+#define MU_DIV_QR_THRESHOLD               1752
+#define MU_DIVAPPR_Q_THRESHOLD            1528
+#define MUPI_DIV_QR_THRESHOLD               82
+#define MU_BDIV_QR_THRESHOLD              1360
 #define MU_BDIV_Q_THRESHOLD               1470
 
-#define MATRIX22_STRASSEN_THRESHOLD         15
-#define HGCD_THRESHOLD                     154
-#define GCD_DC_THRESHOLD                   599
-#define GCDEXT_DC_THRESHOLD                443
-#define JACOBI_BASE_METHOD                   1
+#define POWM_SEC_TABLE  2,17,176,905,2246
 
-#define GET_STR_DC_THRESHOLD                17
-#define GET_STR_PRECOMPUTE_THRESHOLD        34
-#define SET_STR_DC_THRESHOLD               542
-#define SET_STR_PRECOMPUTE_THRESHOLD      1615
+#define MATRIX22_STRASSEN_THRESHOLD         16
+#define HGCD_THRESHOLD                     125
+#define HGCD_APPR_THRESHOLD                143
+#define HGCD_REDUCE_THRESHOLD             4633
+#define GCD_DC_THRESHOLD                   460
+#define GCDEXT_DC_THRESHOLD                330
+#define JACOBI_BASE_METHOD                   4
+
+#define GET_STR_DC_THRESHOLD                15
+#define GET_STR_PRECOMPUTE_THRESHOLD        35
+#define SET_STR_DC_THRESHOLD               272
+#define SET_STR_PRECOMPUTE_THRESHOLD      1183
+
+#define FAC_DSC_THRESHOLD                  336
+#define FAC_ODD_THRESHOLD                   29
