@@ -1,23 +1,21 @@
 /* Test gmp_scanf and related functions.
 
-Copyright 2001, 2002, 2003 Free Software Foundation, Inc.
+Copyright 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 
-This file is part of the GNU MP Library.
+This file is part of the GNU MP Library test suite.
 
-The GNU MP Library is free software; you can redistribute it and/or modify
-it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
-option) any later version.
+The GNU MP Library test suite is free software; you can redistribute it
+and/or modify it under the terms of the GNU General Public License as
+published by the Free Software Foundation; either version 3 of the License,
+or (at your option) any later version.
 
-The GNU MP Library is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
-License for more details.
+The GNU MP Library test suite is distributed in the hope that it will be
+useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
+Public License for more details.
 
-You should have received a copy of the GNU Lesser General Public License
-along with the GNU MP Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+You should have received a copy of the GNU General Public License along with
+the GNU MP Library test suite.  If not, see http://www.gnu.org/licenses/.  */
 
 
 /* Usage: t-scanf [-s]
@@ -28,7 +26,7 @@ MA 02111-1307, USA. */
 
    There's some fairly unattractive repetition between check_z, check_q and
    check_f, but enough differences to make a common loop or a set of macros
-   seem like too much trouble.  */
+   seem like too much trouble. */
 
 
 #include "config.h"
@@ -65,7 +63,7 @@ MA 02111-1307, USA. */
 
 int   option_libc_scanf = 0;
 
-typedef int (*fun_t) _PROTO ((const char *, const char *, void *, void *));
+typedef int (*fun_t) (const char *, const char *, void *, void *);
 
 
 /* This problem was seen on powerpc7450-apple-darwin7.0.0, sscanf returns 0
@@ -231,7 +229,7 @@ fun_sscanf (const char *input, const char *fmt, void *a1, void *a2)
   int     ret;
 
   size = strlen (input) + 1;
-  input_writable = (*__gmp_allocate_func) (size);
+  input_writable = (char *) (*__gmp_allocate_func) (size);
   memcpy (input_writable, input, size);
 
   if (a2 == NULL)
@@ -419,6 +417,14 @@ check_z (void)
     { "%Zd",    "+",    "-999",  0, 1, -555 },
     { "xyz%Zd", "xyz-", "-999",  0, 4, -555 },
     { "xyz%Zd", "xyz+", "-999",  0, 4, -555 },
+    { "%Zi",    "0x",   "-999",  0, 2, -555 },
+    { "%Zi",    "0X",   "-999",  0, 2, -555 },
+    { "%Zi",    "0x-",  "-999",  0, 2, -555 },
+    { "%Zi",    "0X+",  "-999",  0, 2, -555 },
+    { "%Zi",    "-0x",  "-999",  0, 3, -555 },
+    { "%Zi",    "-0X",  "-999",  0, 3, -555 },
+    { "%Zi",    "+0x",  "-999",  0, 3, -555 },
+    { "%Zi",    "+0X",  "-999",  0, 3, -555 },
 
     { "%1Zi",  "1234", "1",    1, 1, 1 },
     { "%2Zi",  "1234", "12",   1, 2, 2 },
@@ -436,7 +442,7 @@ check_z (void)
     { "%7Zi",  "01234", "01234", 1, 5, 5 },
 
     { "%1Zi",  "0x1234", "0",      1, 1, 1 },
-    { "%2Zi",  "0x1234", "0",      1, 2, 2 },
+    { "%2Zi",  "0x1234", "-999",   0, 2, -555 },
     { "%3Zi",  "0x1234", "0x1",    1, 3, 3 },
     { "%4Zi",  "0x1234", "0x12",   1, 4, 4 },
     { "%5Zi",  "0x1234", "0x123",  1, 5, 5 },
@@ -841,12 +847,12 @@ check_q (void)
     { "%9Qi",  "012/057", "012/057",  1, 7 },
 
     { "%1Qi",  "0x12/0x57", "0",         1, 1 },
-    { "%2Qi",  "0x12/0x57", "0",         1, 2 },
+    { "%2Qi",  "0x12/0x57", "-999",      0, 2 },
     { "%3Qi",  "0x12/0x57", "0x1",       1, 3 },
     { "%4Qi",  "0x12/0x57", "0x12",      1, 4 },
-    { "%5Qi",  "0x12/0x57", "-999/121",  0, -1 },
+    { "%5Qi",  "0x12/0x57", "-999/121",  0, 5 },
     { "%6Qi",  "0x12/0x57", "0x12/0",    1, 6 },
-    { "%7Qi",  "0x12/0x57", "0x12/0",    1, 7 },
+    { "%7Qi",  "0x12/0x57", "-999/121",  0, 7 },
     { "%8Qi",  "0x12/0x57", "0x12/0x5",  1, 8 },
     { "%9Qi",  "0x12/0x57", "0x12/0x57", 1, 9 },
     { "%10Qi", "0x12/0x57", "0x12/0x57", 1, 9 },
@@ -1006,7 +1012,7 @@ check_f (void)
     const char  *input;
     const char  *want;
     int         ret;
-    long        ftell;
+    long        ftell;    /* or -1 for length of input string */
 
   } data[] = {
 
@@ -1025,12 +1031,8 @@ check_f (void)
     { "%Ff",  "123.0",  "123", 1, -1 },
     { "%Ff", "+123.0",  "123", 1, -1 },
     { "%Ff", "-123.0", "-123", 1, -1 },
-    { "%Ff",  "123e",   "123", 1, -1 },
-    { "%Ff", "-123e",  "-123", 1, -1 },
-    { "%Ff",  "123e-",  "123", 1, -1 },
-    { "%Ff", "-123e-", "-123", 1, -1 },
-    { "%Ff",  "123e+",  "123", 1, -1 },
-    { "%Ff", "-123e+", "-123", 1, -1 },
+    { "%Ff",  "0123",   "123", 1, -1 },
+    { "%Ff", "-0123",  "-123", 1, -1 },
 
     { "%Ff",  "123.456e3",   "123456", 1, -1 },
     { "%Ff", "-123.456e3",  "-123456", 1, -1 },
@@ -1087,21 +1089,84 @@ check_f (void)
     { "%Ff",    "xyz", "0", 0 },
 
     /* various non-empty but invalid */
-    { "%Ff",    "-",    "-999",  0 },
-    { "%Ff",    "+",    "-999",  0 },
-    { "xyz%Ff", "xyz-", "-999",  0 },
-    { "xyz%Ff", "xyz+", "-999",  0 },
-    { "xyz%Ff", "-.",   "-999",  0 },
-    { "xyz%Ff", "+.",   "-999",  0 },
-    { "xyz%Ff", ".e",   "-999",  0 },
-    { "xyz%Ff", "-.e",  "-999",  0 },
-    { "xyz%Ff", "+.e",  "-999",  0 },
-    { "xyz%Ff", ".E",   "-999",  0 },
-    { "xyz%Ff", "-.E",  "-999",  0 },
-    { "xyz%Ff", "+.E",  "-999",  0 },
-    { "xyz%Ff", ".e123",   "-999",  0 },
-    { "xyz%Ff", "-.e123",  "-999",  0 },
-    { "xyz%Ff", "+.e123",  "-999",  0 },
+    { "%Ff",    "-",      "-999",  0, 1 },
+    { "%Ff",    "+",      "-999",  0, 1 },
+    { "xyz%Ff", "xyz-",   "-999",  0, 4 },
+    { "xyz%Ff", "xyz+",   "-999",  0, 4 },
+    { "%Ff",    "-.",     "-999",  0, 2 },
+    { "%Ff",    "+.",     "-999",  0, 2 },
+    { "%Ff",    ".e",     "-999",  0, 1 },
+    { "%Ff",   "-.e",     "-999",  0, 2 },
+    { "%Ff",   "+.e",     "-999",  0, 2 },
+    { "%Ff",    ".E",     "-999",  0, 1 },
+    { "%Ff",   "-.E",     "-999",  0, 2 },
+    { "%Ff",   "+.E",     "-999",  0, 2 },
+    { "%Ff",    ".e123",  "-999",  0, 1 },
+    { "%Ff",   "-.e123",  "-999",  0, 2 },
+    { "%Ff",   "+.e123",  "-999",  0, 2 },
+    { "%Ff",    "123e",   "-999",  0, 4 },
+    { "%Ff",   "-123e",   "-999",  0, 5 },
+    { "%Ff",    "123e-",  "-999",  0, 5 },
+    { "%Ff",   "-123e-",  "-999",  0, 6 },
+    { "%Ff",    "123e+",  "-999",  0, 5 },
+    { "%Ff",   "-123e+",  "-999",  0, 6 },
+    { "%Ff",   "123e-Z",  "-999",  0, 5 },
+
+    /* hex floats */
+    { "%Ff", "0x123p0",       "291",  1, -1 },
+    { "%Ff", "0x123P0",       "291",  1, -1 },
+    { "%Ff", "0X123p0",       "291",  1, -1 },
+    { "%Ff", "0X123P0",       "291",  1, -1 },
+    { "%Ff", "-0x123p0",     "-291",  1, -1 },
+    { "%Ff", "+0x123p0",      "291",  1, -1 },
+    { "%Ff", "0x123.p0",      "291",  1, -1 },
+    { "%Ff", "0x12.3p4",      "291",  1, -1 },
+    { "%Ff", "-0x12.3p4",    "-291",  1, -1 },
+    { "%Ff", "+0x12.3p4",     "291",  1, -1 },
+    { "%Ff", "0x1230p-4",     "291",  1, -1 },
+    { "%Ff", "-0x1230p-4",   "-291",  1, -1 },
+    { "%Ff", "+0x1230p-4",    "291",  1, -1 },
+    { "%Ff", "+0x.1230p12",   "291",  1, -1 },
+    { "%Ff", "+0x123000p-12", "291",  1, -1 },
+    { "%Ff", "0x123 p12",     "291",  1, 5 },
+    { "%Ff", "0x9 9",           "9",  1, 3 },
+    { "%Ff", "0x01",            "1",  1, 4 },
+    { "%Ff", "0x23",           "35",  1, 4 },
+    { "%Ff", "0x45",           "69",  1, 4 },
+    { "%Ff", "0x67",          "103",  1, 4 },
+    { "%Ff", "0x89",          "137",  1, 4 },
+    { "%Ff", "0xAB",          "171",  1, 4 },
+    { "%Ff", "0xCD",          "205",  1, 4 },
+    { "%Ff", "0xEF",          "239",  1, 4 },
+    { "%Ff", "0xab",          "171",  1, 4 },
+    { "%Ff", "0xcd",          "205",  1, 4 },
+    { "%Ff", "0xef",          "239",  1, 4 },
+    { "%Ff", "0x100p0A",      "256",  1, 7 },
+    { "%Ff", "0x1p9",         "512",  1, -1 },
+
+    /* invalid hex floats */
+    { "%Ff", "0x",     "-999",  0, 2 },
+    { "%Ff", "-0x",    "-999",  0, 3 },
+    { "%Ff", "+0x",    "-999",  0, 3 },
+    { "%Ff", "0x-",    "-999",  0, 2 },
+    { "%Ff", "0x+",    "-999",  0, 2 },
+    { "%Ff", "0x.",    "-999",  0, 3 },
+    { "%Ff", "-0x.",   "-999",  0, 4 },
+    { "%Ff", "+0x.",   "-999",  0, 4 },
+    { "%Ff", "0x.p",   "-999",  0, 3 },
+    { "%Ff", "-0x.p",  "-999",  0, 4 },
+    { "%Ff", "+0x.p",  "-999",  0, 4 },
+    { "%Ff", "0x.P",   "-999",  0, 3 },
+    { "%Ff", "-0x.P",  "-999",  0, 4 },
+    { "%Ff", "+0x.P",  "-999",  0, 4 },
+    { "%Ff", ".p123",  "-999",  0, 1 },
+    { "%Ff", "-.p123", "-999",  0, 2 },
+    { "%Ff", "+.p123", "-999",  0, 2 },
+    { "%Ff", "0x1p",   "-999",  0, 4 },
+    { "%Ff", "0x1p-",  "-999",  0, 5 },
+    { "%Ff", "0x1p+",  "-999",  0, 5 },
+    { "%Ff", "0x123p 12", "291",  0, 6 },
+    { "%Ff", "0x 123p12", "291",  0, 2 },
 
   };
 
@@ -1119,7 +1184,7 @@ check_f (void)
 
   for (i = 0; i < numberof (data); i++)
     {
-      mpf_set_str_or_abort (want, data[i].want, 0);
+      mpf_set_str_or_abort (want, data[i].want, 10);
 
       ASSERT_ALWAYS (strlen (data[i].fmt) + 2 < sizeof (fmt));
       strcpy (fmt, data[i].fmt);
@@ -1137,10 +1202,7 @@ check_f (void)
           want_upto = want_ftell;
 
           if (want_ret == -1 || (want_ret == 0 && ! ignore))
-            {
-              want_ftell = -1;
-              want_upto = -555;
-            }
+            want_upto = -555;
 
           switch (j) {
           case 0:
@@ -1320,7 +1382,7 @@ check_n (void)
     mpq_t  x[2];
     mpq_init (x[0]);
     mpq_init (x[1]);
-    mpq_set_ui (x[0], -987L, 654L);
+    mpq_set_ui (x[0], 987L, 654L);
     mpq_set_ui (x[1], 4115L, 226L);
     ret = gmp_sscanf ("xyz   ", "xyz%Qn", x[0]);
     MPQ_CHECK_FORMAT (x[0]);
@@ -1345,8 +1407,8 @@ check_n (void)
     mpf_t  x[2];
     mpf_init (x[0]);
     mpf_init (x[1]);
-    mpf_set_ui (x[0], -987L);
-    mpf_set_ui (x[1],  654L);
+    mpf_set_ui (x[0], 987L);
+    mpf_set_ui (x[1], 654L);
     ret = gmp_sscanf ("xyz   ", "xyz%Fn", x[0]);
     MPF_CHECK_FORMAT (x[0]);
     MPF_CHECK_FORMAT (x[1]);
@@ -1559,6 +1621,8 @@ main (int argc, char *argv[])
 
   tests_start ();
 
+  mp_trace_base = 16;
+
   check_z ();
   check_q ();
   check_f ();
